#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Copyright  2005 Bill Menees.  All rights reserved.
	Bill@Menees.com

	$History: CommandProcessor.cs $
	
	*****************  Version 1  *****************
	User: Bill         Date: 11/28/07   Time: 9:23p
	Created in $/CSharp/MeneesVSTools/2008
	
	*****************  Version 6  *****************
	User: Bill         Date: 11/21/07   Time: 9:05p
	Updated in $/CSharp/MeneesVSTools/2005
	Recover from VSS crash.
	
	*****************  Version 5  *****************
	User: Bill         Date: 10/30/05   Time: 1:13p
	Updated in $/CSharp/MeneesVSTools/2005
	Updated for VS2005 RTM.
	
	*****************  Version 4  *****************
	User: Bill         Date: 5/08/05    Time: 11:11a
	Updated in $/CSharp/MeneesVSTools/2005
	
	*****************  Version 3  *****************
	User: Bill         Date: 5/08/05    Time: 10:07a
	Updated in $/CSharp/MeneesVSTools/2005
	
	*****************  Version 2  *****************
	User: Bill         Date: 5/08/05    Time: 9:33a
	Updated in $/CSharp/MeneesVSTools/2005
	
	*****************  Version 1  *****************
	User: Bill         Date: 5/01/05    Time: 9:57a
	Created in $/CSharp/MeneesVSTools/2005

-----------------------------------------------------------------------------*/

#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;
using EnvDTE;
using EnvDTE80;
using System.Reflection;

#endregion

namespace Menees.VSTools
{
	class CommandProcessor : IDisposable
	{
		#region Constructor

		public CommandProcessor(DTE2 app, AddIn addIn)
		{
			m_Application = app;
			m_AddIn = addIn;
		}

		#endregion

		#region Public Methods

		public bool CanExecute(string strCommandName)
		{
			bool bResult = false;

			if (strCommandName == "ViewBaseConverter")
			{
				bResult = m_BaseConverterWindow != null;
			}
			else if (strCommandName == "CloseAllExcept")
			{
				bResult = CanCloseAllExcept();
			}
			else
			{
				//All of these commands need a document window to be open.

				Document ActiveDoc = m_Application.ActiveDocument;
				if (ActiveDoc != null)
				{
					//Report each command's status
					switch (strCommandName)
					{
						//These require a text selection.
						case "Sort":
						case "Trim":
						case "Statistics":
						case "RunText":
						case "StreamText":
						case "CheckSpelling":
							bResult = IsTextSelected();
							break;

						//These require a text selection for specific languages.
						case "Comment":
						case "Uncomment":
							bResult = CanCommentSelection(ActiveDoc, strCommandName == "Comment");
							break;

						//This requires an open C# or VB.NET file.
						case "AddRegion":
							switch (Utilities.GetLanguage(ActiveDoc))
							{
								case Language.CSharp:
								case Language.VB:
								case Language.CPlusPlus:
									bResult = true;
									break;
							}
							break;

						//These require an open file.
						case "ExecuteFile":
						case "GenerateGUID":
						case "ToggleReadOnly":
						case "GeneratePrime":
							bResult = true;
							break;

						//This requires a toggleable file.
						case "ToggleFiles":
							switch (Utilities.GetLanguage(ActiveDoc))
							{
								case Language.CPlusPlus:
								case Language.HTML:
								case Language.XML:
								case Language.CSharp:
								case Language.VB:
									bResult = true;
									break;
							}
							break;
					}
				}
			}

			return bResult;
		}

		public bool Execute(string strCommandName)
		{
			bool bResult = true;

			try
			{
				switch (strCommandName)
				{
					case "Sort": Sort(); break;
					case "Trim": Trim(); break;
					case "Statistics": Statistics(); break;
					case "RunText": RunText(); break;
					case "AddRegion": AddRegion(); break;
					case "ExecuteFile": ExecuteFile(); break;
					case "StreamText": StreamText(); break;
					case "GeneratePrime": GeneratePrime(); break;
					case "GenerateGUID": GenerateGUID(); break;
					case "CheckSpelling": CheckSpelling(); break;
					case "ToggleFiles": ToggleFiles(); break;
					case "CloseAllExcept": CloseAllExcept(); break;
					case "ToggleReadOnly": ToggleReadOnly(); break;
					case "Comment": Comment(); break;
					case "Uncomment": Uncomment(); break;
					case "ViewBaseConverter": ViewBaseConverter(); break;
					default:
						{
							string strMessage;
							strMessage = String.Format("'{0}' is not a known {1} command.", strCommandName, m_AddIn.ProgID);
							throw new ArgumentException(strMessage);
						}
				}
			}
			catch (Exception ex)
			{
				string strInnerMsg = "";
				if (ex.InnerException != null)
					strInnerMsg = ex.InnerException.ToString();

				string strMessage = String.Format("Error executing '{0}':\r\n\r\nMessage:  {1}\r\nSource:  {2}\r\nException:  {3}\r\nHelp Link:  {4}\r\nStack Trace:\r\n{5}\r\nInner Exception:  {6}",
					strCommandName, ex.Message, ex.Source, ex.GetType().FullName, ex.HelpLink, ex.StackTrace, strInnerMsg);
				ShowMessage(strMessage, MessageBoxIcon.Error);
				bResult = false;
			}

			return bResult;
		}

		public void CreateToolWindows()
		{
			m_BaseConverterWindow = new BaseConverterToolWindow(m_Application, m_AddIn);
			m_BaseConverterWindow.Initialize();
		}

		#endregion

		#region IDisposable Members

		public void Dispose()
		{
			if (m_BaseConverterWindow != null)
			{
				m_BaseConverterWindow.Close();
			}

			m_Application = null;
			m_AddIn = null;
		}

		#endregion

		#region Private Command Methods

		private void Sort()
		{
			DocInfo Doc;
			if (GetDocInfo(out Doc))
			{
				bool bExecute = true;

				if (!Settings.Default.SortDlgOnlyWhenShift || Utilities.IsShiftPressed())
				{
					SortDlg Dlg = new SortDlg();
					bExecute = Dlg.Execute();
				}

				if (bExecute)
				{
					//Now sort the lines and put them back as the selection
					LineList Lines = new LineList(Doc.SelectedText);
					Lines.Sort(Settings.Default.SortCaseSensitive, Settings.Default.SortOrdinalCompare, Settings.Default.SortAscending, Settings.Default.SortIgnoreWS, Settings.Default.SortEliminateDuplicates);
					Doc.SetSelectedTextIfUnchanged(Lines.ToString());
				}
			}
		}

		private void Trim()
		{
			DocInfo Doc;
			if (GetDocInfo(out Doc))
			{
				bool bExecute = true;

				if (!Settings.Default.TrimDlgOnlyWhenShift || Utilities.IsShiftPressed())
				{
					TrimDlg Dlg = new TrimDlg();
					bExecute = Dlg.Execute();
				}

				if (bExecute && (Settings.Default.TrimLeft || Settings.Default.TrimRight))
				{
					//Now trim the lines and put them back as the selection
					LineList Lines = new LineList(Doc.SelectedText);
					Lines.Trim(Settings.Default.TrimLeft, Settings.Default.TrimRight);
					Doc.SetSelectedTextIfUnchanged(Lines.ToString());
				}
			}
		}

		private void Statistics()
		{
			DocInfo Doc;
			if (GetDocInfo(out Doc))
			{
				StatisticsDlg Dlg = new StatisticsDlg();
				Dlg.Execute(Doc.SelectedText);
			}
		}

		private void RunText()
		{
			DocInfo Doc;
			if (GetDocInfo(out Doc))
			{
				Utilities.ExecuteCmd(Doc.SelectedText, "");
			}
		}

		private void AddRegion()
		{
			DocInfo Doc;
			if (GetDocInfo(out Doc, false))
			{
				RegionBuilder.MakeRegion(m_Application, Doc.Document);
			}
		}

		private void ExecuteFile()
		{
			Document ActiveDoc;
			if (GetActiveDoc(out ActiveDoc))
			{
				bool bCancelled = false;
				bool bSaveAll = Settings.Default.SaveAllBeforeExecute;

				Documents AllDocs = m_Application.Documents;
				if (AllDocs != null && bSaveAll)
				{
					try
					{
						AllDocs.SaveAll();
					}
					catch (ExternalException Ex)
					{
						//Rethrow the exception unless the user hit Cancel
						//when prompted to save changes for a document.
						//Cancelling throws an HRESULT of 0x80004004, which
						//is the C constant E_ABORT with the description
						//"Operation aborted".
						if ((uint)Ex.ErrorCode == 0x80004004)
						{
							bCancelled = true;
						}
						else
						{
							throw;
						}
					}
				}

				if (!bCancelled)
				{
					Utilities.ExecuteCmd(ActiveDoc.FullName, "");
				}
			}
		}

		private void StreamText()
		{
			DocInfo Doc;
			if (GetDocInfo(out Doc))
			{
				LineList Lines = new LineList(Doc.SelectedText);
				Doc.SetSelectedTextIfUnchanged(Lines.Stream());
			}
		}

		private void GeneratePrime()
		{
			DocInfo Doc;
			if (GetDocInfo(out Doc, false))
			{
				int iMaxValue = int.MaxValue;
				string strMessage = String.Format("Enter a number between 1 and {0}.  The next prime number greater than or equal to the input will be returned.", iMaxValue);

				//If there was selected text, and it converts to an int,
				//then use that as the initial value.  Otherwise, use 0,
				//which will show up as a blank input value.
				int iValue = 0;
				string strText = Doc.SelectedText;
				if (!Utilities.IsEmpty(strText))
				{
					int.TryParse(strText, out iValue);
				}

				InputDlg Input = new InputDlg();
				if (Input.GetValue(strMessage, "Generate Prime", 1, iMaxValue, ref iValue))
				{
					iValue = Utilities.GetNextPrime(iValue);
					Doc.SetSelectedText(iValue.ToString());
				}
			}
		}

		private void GenerateGUID()
		{
			DocInfo Doc;
			if (GetDocInfo(out Doc, false))
			{
				Guid guid = Guid.NewGuid();

				string strGUID = guid.ToString();
				strGUID = strGUID.ToUpper();

				//Set the selection to the new GUID
				Doc.SetSelectedText(strGUID);
			}
		}

		private void CheckSpelling()
		{
			DocInfo Doc;
			if (GetDocInfo(out Doc))
			{
				// Launch Word.
				Word._Application App = new Word.ApplicationClass();

				//This is how [optional] VARIANTs have to be passed to .NET wrappers.
				object MissingObj = System.Reflection.Missing.Value;

				// Add a document.
				Word._Document WordDoc = App.Documents.Add(ref MissingObj, ref MissingObj, ref MissingObj, ref MissingObj);

				// Clear current contents.
				Word.Range Range = App.Selection.Range;
				Range.WholeStory();
				Range.Delete(ref MissingObj, ref MissingObj);
				Range = null;

				// Add the text the user selected.
				App.Selection.TypeText(Doc.SelectedText);

				// Show it
				App.Visible = true;

				// Check spelling
				// I originally looked at the App.Options.GetCheckGrammarWithSpelling()
				// flag from Word, but that only works well for streamed text.  For normal
				// source comments with hard-returns, grammar checking stinks because it
				// always warns of fragments.  To allow grammar checking for those that
				// really want it, I provide a global VSTools option to force it.
				if (Settings.Default.CheckGrammarWithSpelling)
					WordDoc.CheckGrammar();
				else
					WordDoc.CheckSpelling(ref MissingObj, ref MissingObj, ref MissingObj, ref MissingObj, ref MissingObj, ref MissingObj, ref MissingObj, ref MissingObj, ref MissingObj, ref MissingObj, ref MissingObj, ref MissingObj);

				// Get the edited text back
				App.Selection.WholeStory();
				string strNewText = App.Selection.Text;

				// Word always adds an extra CR, so strip that off.
				// Also it converts all LFs to CRs, so change
				// that back.
				if (!Utilities.IsEmpty(strNewText))
				{
					if (strNewText.EndsWith("\r"))
						strNewText = strNewText.Substring(0, strNewText.Length - 1);

					strNewText = strNewText.Replace("\r", "\r\n");
				}
				Doc.SetSelectedTextIfUnchanged(strNewText);

				// Tell the doc and Word to go away.
				object SaveChanges = false;
				WordDoc.Close(ref SaveChanges, ref MissingObj, ref MissingObj);
				App.Visible = false;
				App.Quit(ref MissingObj, ref MissingObj, ref MissingObj);
			}
		}

		private void ToggleFiles()
		{
			Document Doc;
			if (GetActiveDoc(out Doc))
			{
				switch (Utilities.GetLanguage(Doc))
				{
					case Language.CPlusPlus:
						ToggleCppDoc(Doc);
						break;
					case Language.HTML:
						ToggleHTMLDoc(Doc);
						break;
					default: //CSharp, Basic, XML, etc.
						ToggleCodeDesigner(Doc);
						break;
				}
			}
		}

		private void CloseAllExcept()
		{
			ExecVSCmd("File.CloseAllButThis");
		}

		private void ToggleReadOnly()
		{
			Document Doc;
			if (GetActiveDoc(out Doc))
			{
				string strName = Doc.FullName;
				FileAttributes Attr = File.GetAttributes(strName);
				if ((int)Attr == -1)
				{
					ShowMessage("File not found: " + strName, MessageBoxIcon.Error);
				}
				else
				{
					if ((Attr & FileAttributes.ReadOnly) == FileAttributes.ReadOnly)
						Attr = Attr & ~FileAttributes.ReadOnly;
					else
						Attr = Attr | FileAttributes.ReadOnly;

					File.SetAttributes(strName, Attr);
				}
			}
		}

		private void Comment()
		{
			CommentSelection(true);
		}

		private void Uncomment()
		{
			CommentSelection(false);
		}

		private void ViewBaseConverter()
		{
			if (m_BaseConverterWindow != null)
			{
				m_BaseConverterWindow.Show();
			}
		}

		#endregion

		#region Private Helper Methods

		private bool GetDocInfo(out DocInfo Info)
		{
			return GetDocInfo(out Info, true);
		}

		private bool GetDocInfo(out DocInfo Info, bool bRequireSelection)
		{
			string strMessage;
			Info = new DocInfo(m_Application, m_AddIn, bRequireSelection, out strMessage);

			if (!Utilities.IsEmpty(strMessage))
			{
				ShowMessage(strMessage, MessageBoxIcon.Warning);
				return false;
			}

			return true;
		}

		private void ShowMessage(string strMessage, MessageBoxIcon Icon)
		{
			if (Settings.Default.ShowWarnings || Icon != MessageBoxIcon.Warning)
			{
				MessageBox.Show(strMessage, m_AddIn.ProgID, MessageBoxButtons.OK, Icon);
			}
		}

		private bool GetActiveDoc(out Document Doc)
		{
			bool bResult = true;

			Doc = m_Application.ActiveDocument;
			if (Doc == null)
			{
				ShowMessage("There is no active document.", MessageBoxIcon.Warning);
				bResult = false;
			}

			return bResult;
		}

		private bool IsExtension(string strExt, string[] arExtensions)
		{
			bool bResult = false;

			for (int i = 0; i < arExtensions.Length; i++)
			{
				if (String.Compare(strExt, arExtensions[i], true) == 0)
				{
					bResult = true;
					break;
				}
			}

			return bResult;
		}

		private void OpenFileWithExt(string strName, string[] arExtensions, string strBasePath, string strOtherDirs)
		{
			//See if file is currently open and activate it.
			//This is faster than hitting the disk, and if
			//the user has manually opened a file with a "later"
			//extension, it will be found before we try to
			//open earlier extensions from the disk.
			foreach (string strExt in arExtensions)
			{
				Document Doc = GetOpenDoc(strName, strExt);
				if (Doc != null)
				{
					//The IDE sometimes thinks it has a document open even
					//if it has no windows.  I've seen cases where I manually
					//close a file, but it is still in the Documents collection.
					//I think this is an IDE bug because it rarely happens.
					//
					//I'll work around it by checking to see if the document
					//still has any windows.  If not, then we'll try to create
					//a new window before activating it.  If we try to Activate 
					//a document with no windows the IDE throws an exception.
					//
					//If this fails, I'll just eat the exception and hope we
					//can open the file from disk.
					try
					{
						if (Doc.Windows.Count == 0)
						{
							Doc.NewWindow();
						}

						Doc.Activate();

						//If we got here then everything succeeded.
						//We activated the correct document, so we're done.
						return;
					}
					catch (Exception)
					{
						//Ignore the error and continue.
					}
				}
			}

			//There's no matching file open, so try to find one 
			//on disk in the base path and open it.
			if (SearchDirForFileWithExt(strBasePath, strName, arExtensions))
			{
				return;
			}

			//See if there are any additional directories to search.
			if (!Utilities.IsEmpty(strOtherDirs))
			{
				//Split the semicolon or comma delimited list of paths
				char[] arSeparators = new Char[] { ';', ',' };
				string[] arDirectories = strOtherDirs.Split(arSeparators);

				//Search each path for the file
				foreach (string strDir in arDirectories)
				{
					string strPath = strDir.Trim();

					//Strip double quotes off
					if (strPath.StartsWith("\"")) strPath = strPath.Length > 0 ? strPath.Substring(1) : "";
					if (strPath.EndsWith("\"")) strPath = strPath.Length > 0 ? strPath.Substring(0, strPath.Length - 1) : "";

					if (!strPath.EndsWith("\\")) strPath += "\\";

					if (SearchDirForFileWithExt(strPath, strName, arExtensions))
					{
						return;
					}
				}
			}
		}

		private bool ExecVSCmd(string strCmd)
		{
			return ExecVSCmd(strCmd, "");
		}

		private bool ExecVSCmd(string strCmd, string strArgs)
		{
			bool bResult = false;

			Command Cmd = m_Application.Commands.Item(strCmd, 0);
			if (Cmd.IsAvailable)
			{
				m_Application.ExecuteCommand(strCmd, strArgs);
				bResult = true;
			}

			return bResult;
		}

		private bool CanExecVSCmd(string strCmd)
		{
			Command Cmd = m_Application.Commands.Item(strCmd, 0);
			bool bResult = Cmd.IsAvailable;
			return bResult;
		}

		private bool SearchDirForFileWithExt(string strPath, string strName, string[] arExtensions)
		{
			foreach (string strExt in arExtensions)
			{
				string strFullName = Path.Combine(strPath, strName + strExt);
				if (File.Exists(strFullName))
				{
					string strQuotedFileName = String.Format("\"{0}\"", strFullName);
					return ExecVSCmd("File.OpenFile", strQuotedFileName);
				}
			}

			return false;
		}

		private Document GetOpenDoc(string strName, string strExt)
		{
			Document docResult = null;
			string strNameWithExt = strName + strExt;

			foreach (Document Doc in m_Application.Documents)
			{
				if (String.Compare(Path.GetFileName(Doc.FullName), strNameWithExt, true) == 0)
				{
					docResult = Doc;
					break;
				}
			}

			return docResult;
		}

		private void ToggleCppDoc(Document Doc)
		{
			string strFullName = Doc.FullName;
			string strExt = Path.GetExtension(strFullName);
			if (strExt.Length > 0)
			{
				string strName = Path.GetFileNameWithoutExtension(strFullName);
				string strPath = Path.GetDirectoryName(strFullName);
				string[] arExtensions = null;

				if (IsExtension(strExt, m_arCppHeaderExtensions))
				{
					arExtensions = m_arCppImplementationExtensions;
				}
				else if (IsExtension(strExt, m_arCppImplementationExtensions))
				{
					arExtensions = m_arCppHeaderExtensions;
				}

				if (arExtensions != null)
				{
					OpenFileWithExt(strName, arExtensions, strPath, Settings.Default.CppDirectories);
				}
			}
		}

		private void ToggleCodeDesigner(Document Doc)
		{
			//If a non-document window is selected (e.g. Solution Explorer, 
			//Properties, etc), then try to force a document window to be
			//activated.
			ExecVSCmd("Window.ActivateDocumentWindow");

			//If the active window is a document, then activate
			//the first document window that is NOT equal to the
			//active window.
			Window ActiveWin = m_Application.ActiveWindow;
			if (ActiveWin != null && ActiveWin.Type == vsWindowType.vsWindowTypeDocument)
			{
				foreach (Window Win in Doc.Windows)
				{
					if (!ActiveWin.Equals(Win))
					{
						Win.Activate();
						return;
					}
				}

				//The active window is a text document, but the
				//active document's window was equal to the active
				//window.  So we'll try the View.ViewCode command
				//first.  If it doesn't change the active window,
				//then we'll try View.ViewDesigner.
				//
				//These commands aren't available when a solution 
				//isn't open, and they may not be available at other
				//times.  So this may do nothing, but it's our best
				//shot...
				ExecVSCmd("View.ViewCode");
				if (m_Application.ActiveWindow.Equals(ActiveWin))
				{
					// The document may not have anything to design.
					ExecVSCmd("View.ViewDesigner");
				}
			}
		}

		private void ToggleHTMLDoc(Document Doc)
		{
			//Execute the NextView command.  It may fail for
			//non-designable HTML docs (e.g. XSLT).
			if (ExecVSCmd("View.NextView"))
			{
				//If it succeeded then we're done.
				return;
			}

			//If NextView failed, then try to manually change the tab.
			Window Win = Doc.ActiveWindow;
			if (Win != null)
			{
				object objWin = Win.Object;
				if (objWin is HTMLWindow)
				{
					HTMLWindow HTML = (HTMLWindow)objWin;
					vsHTMLTabs tabCurrent = HTML.CurrentTab;
					if (tabCurrent == vsHTMLTabs.vsHTMLTabsSource)
						HTML.CurrentTab = vsHTMLTabs.vsHTMLTabsDesign;
					else
						HTML.CurrentTab = vsHTMLTabs.vsHTMLTabsSource;
				}
			}
		}

		private bool CanCommentSelection(Document ActiveDoc, bool bComment)
		{
			//First make sure text is selected.  The VS commands don't require that,
			//but visually I don't want the toolbar buttons enabled unless text is
			//selected.
			bool bResult = IsTextSelected();
			if (bResult)
			{
				bResult = CanExecVSCmd(bComment ? "Edit.CommentSelection" : "Edit.UncommentSelection");

				//The VS command status will be correct for everything except SQL files.
				//For SQL files, we'll do the commenting ourselves.
				if (!bResult && Utilities.GetLanguage(ActiveDoc) == Language.SQL)
				{
					bResult = true;
				}
			}

			return bResult;
		}

		private void CommentSelection(bool bComment)
		{
			DocInfo Doc;
			if (GetDocInfo(out Doc))
			{
				//In VS2005, the Edit.CommentSelection and Edit.UncommentSelection
				//commands do 99% of what I originally wrote this method for, so
				//I'll use them whenever possible.  However, they don't handle SQL
				//commenting, and they don't provide an option for multi-line comments
				//in languages that support single and multi-line comments.  For those
				//cases, I'll still use my commenting methods.
				bool bUseVSCommand = true;

				bool bUseMultiLine = Settings.Default.MultiLineComment;

				string strSingle = "";
				string strMultiStart = "";
				string strMultiFinish = "";

				switch (Utilities.GetLanguage(Doc.Document))
				{
					case Language.CPlusPlus:
					case Language.CSharp:
					case Language.IDL:
					case Language.JScript:
						bUseVSCommand = !bUseMultiLine;
						strSingle = "//";
						strMultiStart = "/*";
						strMultiFinish = "*/";
						break;
					case Language.VB:
					case Language.VBScript:
						strSingle = "'";
						bUseMultiLine = false;
						break;
					case Language.HTML:
					case Language.XML:
						strMultiStart = "<!--";
						strMultiFinish = "-->";
						bUseMultiLine = true;
						break;
					case Language.SQL:
						bUseVSCommand = false;
						strSingle = "--";
						strMultiStart = "/*";
						strMultiFinish = "*/";
						break;
				}

				if (strSingle.Length > 0 || strMultiStart.Length > 0)
				{
					if (bUseVSCommand)
					{
						ExecVSCmd(bComment ? "Edit.CommentSelection" : "Edit.UncommentSelection");
					}
					else
					{
						LineList Lines = new LineList(Doc.SelectedText);
						if (bUseMultiLine)
						{
							Lines.Comment(bComment, strMultiStart, strMultiFinish);
						}
						else
						{
							Lines.Comment(bComment, strSingle);
						}
						Doc.SetSelectedTextIfUnchanged(Lines.ToString());
					}
				}
			}
		}

		private bool CanCloseAllExcept()
		{
			bool bResult = CanExecVSCmd("File.CloseAllButThis");
			return bResult;
		}

		private bool IsTextSelected()
		{
			bool bResult = false;

			Document ActiveDoc = m_Application.ActiveDocument;
			if (ActiveDoc != null && ActiveDoc.Selection != null)
			{
				TextDocument TextDoc = (TextDocument)ActiveDoc.Object("TextDocument");
				TextSelection Sel = TextDoc.Selection;
				bResult = (Sel != null) && !Sel.IsEmpty;
			}

			return bResult;
		}

		#endregion
   
		#region Private Data Members

		private DTE2 m_Application;
		private AddIn m_AddIn;
		private ToolWindow m_BaseConverterWindow;

		private readonly string[] m_arCppHeaderExtensions = { ".h", ".tlh", ".hpp", ".hxx", ".hh" };
		private readonly string[] m_arCppImplementationExtensions = { ".cpp", ".c", ".inl", ".tli", ".cxx", ".cc" };

		#endregion
	}
}
