#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Copyright  2005 Bill Menees.  All rights reserved.
	Bill@Menees.com

	$History: DocInfo.cs $
	
	*****************  Version 1  *****************
	User: Bill         Date: 11/28/07   Time: 9:23p
	Created in $/CSharp/MeneesVSTools/2008
	
	*****************  Version 1  *****************
	User: Bill         Date: 5/01/05    Time: 9:57a
	Created in $/CSharp/MeneesVSTools/2005

-----------------------------------------------------------------------------*/

#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using EnvDTE;

#endregion

namespace Menees.VSTools
{
	internal class DocInfo
	{
		#region Constructors

		public DocInfo(_DTE App, AddIn _AddIn, bool bRequireSelection, out string strErrorMessage)
		{
			strErrorMessage = null;
			m_Application = App;
			m_AddIn = _AddIn;
			m_Doc = m_Application.ActiveDocument;
			if (m_Doc != null)
			{
				m_TextDoc = (TextDocument)m_Doc.Object("TextDocument");
				if (m_TextDoc != null)
				{
					/*
						I originally checked m_TextDoc.ReadOnly before write operations
						because it seemed like a good idea to fail early with an explicit error
						message.  However, this didn't work well with Visual SourceSafe integration.

						By default, if you don't have a file checked out, it is read-only.  But when
						you try to type, paste, or set the Selection.Text property in any way, you
						get a confirmation box asking you if you want to check the file out.  To work
						with this useful feature, I have to allow getting the selection from a read-only
						document even for write operations.  The write may or may not succeed, but
						that has to be determined outside the scope of this function.
					*/

					m_TextSel = m_TextDoc.Selection;
					if (m_TextSel != null)
					{
						//Always get the selected text, even if it
						//isn't required.  SetSelectedTextIfUnchanged may need
						//to check for differences.
						m_strSelectedText = m_TextSel.Text;
						if (bRequireSelection && m_strSelectedText.Length == 0)
						{
							strErrorMessage = "No text is selected.";
						}
					}
					else
					{
						strErrorMessage = "Unable to get selection object.";
					}
				}
				else
				{
					strErrorMessage = "The active document is not a text document.";
				}
			}
			else
			{
				strErrorMessage = "There is no active document.";
			}
		}

		#endregion

		#region Public Properties

		public string SelectedText
		{
			get
			{
				return m_strSelectedText;
			}
		}

		public Document Document
		{
			get
			{
				return m_Doc;
			}
		}

		#endregion

		#region Public Methods

		public void SetSelectedTextIfUnchanged(string strNewText)
		{
			//Check for differences, and if there are no changes then
			//don't overwrite the selection.  This keeps the document
			//from being marked Modified if no modifications have been
			//made.
			if (m_strSelectedText != strNewText)
			{
				SetSelectedText(strNewText, true);
			}
		}

		public void SetSelectedText(string strNewText)
		{
			SetSelectedText(strNewText, false);
		}

		public void SetSelectedText(string strNewText, bool bKeepSelection)
		{
			//Put all of the changes in a single Undo context.
			UndoContext Undo = m_Application.UndoContext;
			bool bUndoAlreadyOpen = Undo.IsOpen;

			if (!bUndoAlreadyOpen)
			{
				Undo.Open(m_AddIn.ProgID + ".SetSelectedText", false);
			}

			try
			{
				//Don't set the TextSel.Text property.  It behaves weirdly.
				//It behaves like a bunch of individual line inserts, and
				//then the editor's indenting can really screw things up,
				//especially for text files with Block Indent (the default).
			
				//Delete any existing selected text.
				if (!m_TextSel.IsEmpty)
				{
					//If we pass zero here, then nothing happens.
					//It appears to work fine (i.e. deleting the 
					//entire selection) if we pass any non-zero 
					//value.
					m_TextSel.Delete(1);
				}

				//Insert the new text, optionally drop the selection, 
				//and leave the caret at the end.
				if (bKeepSelection)
				{
					m_TextSel.Insert(strNewText, (int)vsInsertFlags.vsInsertFlagsInsertAtStart);
					m_TextSel.SwapAnchor();
				}
				else
				{
					m_TextSel.Insert(strNewText, (int)vsInsertFlags.vsInsertFlagsCollapseToEnd);
				}
			}
			finally
			{
				//Make sure we close the UndoContext.
				if (!bUndoAlreadyOpen)
				{
					Undo.Close();
				}
			}
		}

		#endregion

		#region Private Data Members

		private Document m_Doc;
		private TextDocument m_TextDoc;
		private TextSelection m_TextSel;
		private string m_strSelectedText;
		private _DTE m_Application;
		private AddIn m_AddIn;

		#endregion
	}
}
