#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Copyright  2007 Bill Menees.  All rights reserved.
	Bill@Menees.com

	$History: Number.cs $
	
	*****************  Version 1  *****************
	User: Bill         Date: 11/28/07   Time: 9:23p
	Created in $/CSharp/MeneesVSTools/2008
	
	*****************  Version 1  *****************
	User: Bill         Date: 3/12/07    Time: 10:50a
	Created in $/CSharp/MeneesVSTools/2005
	Updated to v1.1.

-----------------------------------------------------------------------------*/

#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Globalization;

#endregion

namespace Menees.VSTools
{
	internal sealed class Number
	{
		#region Constructors

		public Number(string textValue, NumberBase numBase, ByteOrder byteOrder, DataType dataType)
		{
			if (!string.IsNullOrEmpty(textValue))
			{
				try
				{
					switch (numBase)
					{
						case NumberBase.Binary:
							InitializeFromBinary(textValue, byteOrder, dataType);
							break;
						case NumberBase.Decimal:
							InitializeFromDecimal(textValue, byteOrder, dataType);
							break;
						case NumberBase.Hex:
							InitializeFromHex(textValue, byteOrder, dataType);
							break;
					}
				}
				catch (Exception)
				{
					//Ignore errors since I allow arbitrary input.
				}
			}
		}

		#endregion

		#region Public Properties

		public string BinaryValue
		{
			get
			{
				return m_binaryValue;
			}
		}

		public string DecimalValue
		{
			get
			{
				return m_decimalValue;
			}
		}

		public string HexValue
		{
			get
			{
				return m_hexValue;
			}
		}

		#endregion

		#region Private Methods

		private void InitializeFromDecimal(string textValue, ByteOrder byteOrder, DataType dataType)
		{
			const NumberStyles c_integerStyles = NumberStyles.Integer | NumberStyles.AllowThousands;

			byte[] machineValueBytes = null;
			switch (dataType)
			{
				case DataType.Byte:
					byte byteValue;
					if (byte.TryParse(textValue, c_integerStyles, null, out byteValue))
					{
						machineValueBytes = new byte[1] { byteValue };
					}
					break;
				case DataType.Short:
					short shortValue;
					if (short.TryParse(textValue, c_integerStyles, null, out shortValue))
					{
						machineValueBytes = BitConverter.GetBytes(shortValue);
					}
					break;
				case DataType.Int:
					int intValue;
					if (int.TryParse(textValue, c_integerStyles, null, out intValue))
					{
						machineValueBytes = BitConverter.GetBytes(intValue);
					}
					break;
				case DataType.Long:
					long longValue;
					if (long.TryParse(textValue, c_integerStyles, null, out longValue))
					{
						machineValueBytes = BitConverter.GetBytes(longValue);
					}
					break;
				case DataType.Single:
					float singleValue;
					if (float.TryParse(textValue, out singleValue))
					{
						machineValueBytes = BitConverter.GetBytes(singleValue);
					}
					break;
				case DataType.Double:
					double doubleValue;
					if (double.TryParse(textValue, out doubleValue))
					{
						machineValueBytes = BitConverter.GetBytes(doubleValue);
					}
					break;
				case DataType.Decimal:
					decimal decimalValue;
					if (decimal.TryParse(textValue, out decimalValue))
					{
						machineValueBytes = GetBytes(decimalValue);
					}
					break;
				case DataType.SignedByte:
					sbyte sbyteValue;
					if (sbyte.TryParse(textValue, c_integerStyles, null, out sbyteValue))
					{
						unchecked
						{
							byte castValue = (byte)sbyteValue;
							machineValueBytes = new byte[1] { castValue };
						}
					}
					break;
				case DataType.UnsignedShort:
					ushort ushortValue;
					if (ushort.TryParse(textValue, c_integerStyles, null, out ushortValue))
					{
						machineValueBytes = BitConverter.GetBytes(ushortValue);
					}
					break;
				case DataType.UnsignedInt:
					uint uintValue;
					if (uint.TryParse(textValue, c_integerStyles, null, out uintValue))
					{
						machineValueBytes = BitConverter.GetBytes(uintValue);
					}
					break;
				case DataType.UnsignedLong:
					ulong ulongValue;
					if (ulong.TryParse(textValue, c_integerStyles, null, out ulongValue))
					{
						machineValueBytes = BitConverter.GetBytes(ulongValue);
					}
					break;
			}

			if (machineValueBytes != null)
			{
				m_decimalValue = textValue;
				byte[] requestedOrderBytes = EnsureRequestedByteOrder(machineValueBytes, byteOrder);
				m_hexValue = GenerateHexString(requestedOrderBytes, byteOrder);
				m_binaryValue = GenerateBinaryString(requestedOrderBytes, byteOrder);
			}
		}

		private void InitializeFromBinary(string textValue, ByteOrder byteOrder, DataType dataType)
		{
			int requiredNumBytes = GetNumBytesForDataType(dataType);
			byte[] machineValueBytes = GetMachineBytesFromText(textValue, byteOrder, NumberBase.Binary, requiredNumBytes);
			object convertedValue = GetConvertedValueForBytes(machineValueBytes, dataType);

			if (convertedValue != null)
			{
				m_decimalValue = FormatDecimalValue(convertedValue);
				m_binaryValue = textValue;
				byte[] requestedOrderBytes = EnsureRequestedByteOrder(machineValueBytes, byteOrder);
				m_hexValue = GenerateHexString(requestedOrderBytes, byteOrder);
			}
		}

		private void InitializeFromHex(string textValue, ByteOrder byteOrder, DataType dataType)
		{
			int requiredNumBytes = GetNumBytesForDataType(dataType);
			byte[] machineValueBytes = GetMachineBytesFromText(textValue, byteOrder, NumberBase.Hex, requiredNumBytes);
			object convertedValue = GetConvertedValueForBytes(machineValueBytes, dataType);

			if (convertedValue != null)
			{
				m_decimalValue = FormatDecimalValue(convertedValue);
				m_hexValue = textValue;
				byte[] requestedOrderBytes = EnsureRequestedByteOrder(machineValueBytes, byteOrder);
				m_binaryValue = GenerateBinaryString(requestedOrderBytes, byteOrder);
			}
		}

		private string FormatDecimalValue(object convertedValue)
		{
			string formatStr = "{0}";
			if (Settings.Default.NumberUseSystemNumberFmtForDecimal)
			{
				TypeCode code = Type.GetTypeCode(convertedValue.GetType());
				//I'm only doing this for integral types because I can't find a way to
				//combine "General" and "Number" formats for floating-point types.
				//I want all significant digits after the decimal, but I also want the
				//group delimiters in front of the decimal.
				if (code != TypeCode.Single && code != TypeCode.Double && code != TypeCode.Decimal)
				{
					formatStr = "{0:N0}";
				}
			}

			string result = string.Format(formatStr, convertedValue);
			return result;
		}

		private static object GetConvertedValueForBytes(byte[] machineValueBytes, DataType dataType)
		{
			object convertedValue = null;

			if (machineValueBytes != null)
			{
				switch (dataType)
				{
					case DataType.Byte:
						convertedValue = machineValueBytes[0];
						break;
					case DataType.Decimal:
						convertedValue = ToDecimal(machineValueBytes);
						break;
					case DataType.Double:
						convertedValue = BitConverter.ToDouble(machineValueBytes, 0);
						break;
					case DataType.Int:
						convertedValue = BitConverter.ToInt32(machineValueBytes, 0);
						break;
					case DataType.Long:
						convertedValue = BitConverter.ToInt64(machineValueBytes, 0);
						break;
					case DataType.Short:
						convertedValue = BitConverter.ToInt16(machineValueBytes, 0);
						break;
					case DataType.SignedByte:
						unchecked
						{
							convertedValue = (sbyte)machineValueBytes[0];
						}
						break;
					case DataType.Single:
						convertedValue = BitConverter.ToSingle(machineValueBytes, 0);
						break;
					case DataType.UnsignedInt:
						convertedValue = BitConverter.ToUInt32(machineValueBytes, 0);
						break;
					case DataType.UnsignedLong:
						convertedValue = BitConverter.ToUInt64(machineValueBytes, 0);
						break;
					case DataType.UnsignedShort:
						convertedValue = BitConverter.ToUInt16(machineValueBytes, 0);
						break;
				}
			}

			return convertedValue;
		}

		private static decimal ToDecimal(byte[] bytes)
		{
			//From Nick Darnell on http://msdn2.microsoft.com/en-us/library/system.bitconverter_methods.aspx
			int[] bits = new int[4];

			unchecked
			{
				bits[0] = ((bytes[0] | (bytes[1] << 8)) | (bytes[2] << 0x10)) | (bytes[3] << 0x18); //lo
				bits[1] = ((bytes[4] | (bytes[5] << 8)) | (bytes[6] << 0x10)) | (bytes[7] << 0x18); //mid
				bits[2] = ((bytes[8] | (bytes[9] << 8)) | (bytes[10] << 0x10)) | (bytes[11] << 0x18); //hi
				bits[3] = ((bytes[12] | (bytes[13] << 8)) | (bytes[14] << 0x10)) | (bytes[15] << 0x18); //flags
			}

			return new decimal(bits);
		}

		private static byte[] GetBytes(decimal d)
		{
			//From Nick Darnell on http://msdn2.microsoft.com/en-us/library/system.bitconverter_methods.aspx
			byte[] bytes = new byte[16]; 

			int[] bits = decimal.GetBits(d); 
			int lo = bits[0]; 
			int mid = bits[1]; 
			int hi = bits[2]; 
			int flags = bits[3]; 
			bytes[0] = (byte)lo; 
			bytes[1] = (byte)(lo >> 8); 
			bytes[2] = (byte)(lo >> 0x10); 
			bytes[3] = (byte)(lo >> 0x18); 
			bytes[4] = (byte)mid; 
			bytes[5] = (byte)(mid >> 8); 
			bytes[6] = (byte)(mid >> 0x10); 
			bytes[7] = (byte)(mid >> 0x18); 
			bytes[8] = (byte)hi; 
			bytes[9] = (byte)(hi >> 8); 
			bytes[10] = (byte)(hi >> 0x10); 
			bytes[11] = (byte)(hi >> 0x18); 
			bytes[12] = (byte)flags; 
			bytes[13] = (byte)(flags >> 8); 
			bytes[14] = (byte)(flags >> 0x10); 
			bytes[15] = (byte)(flags >> 0x18); 

			return bytes;
		}

		private string GenerateBinaryString(byte[] requestedOrderBytes, ByteOrder byteOrder)
		{
			StringBuilder sb = new StringBuilder(8 * requestedOrderBytes.Length);

			foreach (byte currentByte in requestedOrderBytes)
			{
				if (sb.Length > 0 && Settings.Default.NumberUseSpaceSeparators)
				{
					sb.Append(' ');
				}

				for (int i = 7; i >= 0; i--)
				{
					bool isBitSet = (currentByte & (1 << i)) != 0;
					sb.Append(isBitSet ? "1" : "0");
				}
			}

			string result = sb.ToString();
			if (ShouldTrimLeadingZeros(byteOrder))
			{
				result = TrimLeadingZeros(result);
			}
			return result;
		}

		private string GenerateHexString(byte[] requestedOrderBytes, ByteOrder byteOrder)
		{
			StringBuilder sb = new StringBuilder(2*requestedOrderBytes.Length);

			foreach (byte b in requestedOrderBytes)
			{
				if (sb.Length > 0 && Settings.Default.NumberUseSpaceSeparators)
				{
					sb.Append(' ');
				}
				sb.AppendFormat("{0:X2}", b);
			}

			string result = sb.ToString();
			if (ShouldTrimLeadingZeros(byteOrder))
			{
				result = TrimLeadingZeros(result);
			}
			return result;
		}

		private bool ShouldTrimLeadingZeros(ByteOrder byteOrder)
		{
			if (byteOrder == ByteOrder.Numeric)
			{
				return Settings.Default.NumberTrimLeadingZerosNumeric;
			}
			else
			{
				return Settings.Default.NumberTrimLeadingZerosEndian;
			}
		}

		private string TrimLeadingZeros(string result)
		{
			int nonZeroCharPos = -1;
			for (int i = 0; i < result.Length; i++)
			{
				char ch = result[i];
				if (ch != '0' && ch != ' ')
				{
					nonZeroCharPos = i;
					break;
				}
			}

			if (nonZeroCharPos > 0)
			{
				result = result.Substring(nonZeroCharPos);
			}
			else if (nonZeroCharPos < 0)
			{
				result = "0";
			}

			return result;
		}

		private byte[] GetMachineBytesFromText(string textValue, ByteOrder byteOrder, NumberBase numBase, int requiredNumBytes)
		{
			Debug.Assert(numBase == NumberBase.Binary || numBase == NumberBase.Hex);

			byte[] result = null;
			bool validLength = ValidateAndNormalizeText(ref textValue, byteOrder, numBase, requiredNumBytes);
			if (validLength)
			{
				string[] byteStrings = SplitIntoByteStrings(textValue, numBase == NumberBase.Binary ? 8 : 2);
				List<byte> byteList = new List<byte>(requiredNumBytes);
				foreach (string byteString in byteStrings)
				{
					byte byteValue;
					if (TryParse(byteString, numBase, out byteValue))
					{
						byteList.Add(byteValue);
					}
					else
					{
						return null;
					}
				}

				result = byteList.ToArray();

				//Regardless of input byte order we must return bytes that match machine byte order
				//(i.e., BitConverter.IsLittleEndian) because our output will be passed to BitConverter.
				result = EnsureRequestedByteOrder(result, byteOrder);
			}

			return result;
		}

		private bool TryParse(string byteString, NumberBase numBase, out byte byteValue)
		{
			bool result;

			if (numBase == NumberBase.Hex)
			{
				result = byte.TryParse(byteString, NumberStyles.HexNumber, null, out byteValue);
			}
			else
			{
				result = true;
				byteValue = 0;
				for (int i = 7; i >= 0; i--)
				{
					char ch = byteString[7 - i];
					if (ch == '1')
					{
						byteValue |= (byte)(1 << i);
					}
					else if (ch != '0')
					{
						result = false;
						break;
					}
				}
			}

			return result;
		}

		private string[] SplitIntoByteStrings(string textValue, int numCharsPerByte)
		{
			Debug.Assert(textValue.Length % numCharsPerByte == 0);

			int numBytes = textValue.Length / numCharsPerByte;
			List<string> byteStrings = new List<string>(numBytes);
			for (int groupIndex = 0; groupIndex < numBytes; groupIndex++)
			{
				string byteString = textValue.Substring(groupIndex * numCharsPerByte, numCharsPerByte);
				byteStrings.Add(byteString);
			}

			return byteStrings.ToArray();
		}

		private static bool ValidateAndNormalizeText(ref string textValue, ByteOrder byteOrder, NumberBase numBase, int requiredNumBytes)
		{
			//Ignore 0x prefix for hex
			if (numBase == NumberBase.Hex && textValue.StartsWith("0x", StringComparison.OrdinalIgnoreCase))
			{
				textValue = textValue.Substring(2);
			}

			//Ignore spaces
			textValue = textValue.Replace(" ", "");

			//For big and little endian, the number of bytes MUST match the required number to
			//avoid ambiguities with zero padding (which end would we pick?).  For numeric order,
			//we can pad up to the number of required bytes, but we can never truncate.
			int numCharsPerByte = numBase == NumberBase.Binary ? 8 : 2;
			int numCharsTotal = textValue.Length;
			bool validLength = false;
			if (byteOrder == ByteOrder.Numeric)
			{
				//I'm depending on the truncation of integer division here.
				int numWholeBytes = numCharsTotal / numCharsPerByte;
				int numLeftoverCharacters = numCharsTotal % numCharsPerByte;
				if (numWholeBytes == requiredNumBytes && numLeftoverCharacters == 0)
				{
					validLength = true;
				}
				else if (numWholeBytes < requiredNumBytes)
				{
					validLength = true;
					int numPadZeros = (requiredNumBytes - numWholeBytes - 1) * numCharsPerByte + (numCharsPerByte - numLeftoverCharacters);
					textValue = new string('0', numPadZeros) + textValue;
				}
			}
			else
			{
				validLength = numCharsTotal % numCharsPerByte == 0 && numCharsTotal / numCharsPerByte == requiredNumBytes;
			}

			return validLength;
		}

		private byte[] EnsureRequestedByteOrder(byte[] machineValueBytes, ByteOrder byteOrder)
		{
			if (!MachineOrderMatchesByteOrder(byteOrder))
			{
				Array.Reverse(machineValueBytes);
			}

			return machineValueBytes;
		}

		private bool MachineOrderMatchesByteOrder(ByteOrder byteOrder)
		{
			//As far as this function is concerned, numeric and big endian are the same because they
			//both order from most significant down to least significant.
			bool result =	(BitConverter.IsLittleEndian && byteOrder == ByteOrder.LittleEndian) ||
								(!BitConverter.IsLittleEndian && byteOrder != ByteOrder.LittleEndian);
			return result;
		}

		private int GetNumBytesForDataType(DataType dataType)
		{
			switch (dataType)
			{
				case DataType.Byte:
				case DataType.SignedByte:
					return 1;

				case DataType.Short:
				case DataType.UnsignedShort:
					return 2;

				case DataType.Int:
				case DataType.UnsignedInt:
				case DataType.Single:
					return 4;

				case DataType.Double:
				case DataType.Long:
				case DataType.UnsignedLong:
					return 8;

				case DataType.Decimal:
					return 16;
			}

			return 0;
		}

		#endregion

		#region Private Data Members

		private string m_binaryValue = "";
		private string m_decimalValue = "";
		private string m_hexValue = "";

		#endregion
	}
}
