#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Copyright  2005 Bill Menees.  All rights reserved.
	Bill@Menees.com

	$History: RegionBuilder.cs $
	
	*****************  Version 1  *****************
	User: Bill         Date: 11/28/07   Time: 9:23p
	Created in $/CSharp/MeneesVSTools/2008
	
	*****************  Version 1  *****************
	User: Bill         Date: 5/08/05    Time: 9:33a
	Created in $/CSharp/MeneesVSTools/2005

-----------------------------------------------------------------------------*/

#endregion

#region Using Directives

using System;
using System.Text;
using System.Diagnostics; 
using System.Collections.Generic;
using EnvDTE;
using EnvDTE80;

#endregion

namespace Menees.VSTools
{
	//Note: This class started life as a macro from:
	//http://www.componentscience.net/csblog/PermaLink.aspx?guid=ee1ff669-60b1-400f-bbb5-04f83ebe04f4
	//But then I modified it heavily to use my formatting, to work with Option Strict On, and to provide
	//standard region names.  Then after it was tested and debugged I converted it to C# using:
	//http://www.developerfusion.com/utilities/convertvbtocsharp.aspx.
	//Then I tweaked it a little more to use a configurable list of predefined regions.
	internal static class RegionBuilder
	{
		#region Public Methods

		public static bool MakeRegion(DTE2 App, Document Doc)
		{
			Language eLanguage = Utilities.GetLanguage(Doc);
			if (eLanguage != Language.CSharp && eLanguage != Language.VB && eLanguage != Language.CPlusPlus)
			{
				//MakeRegion can only be used with C#, VB, and C++ documents.
				return false;
			}

			TextSelection sel = ((TextSelection)(Doc.Selection));
			EditPoint startpoint = sel.TopPoint.CreateEditPoint();
			EditPoint endpoint = sel.BottomPoint.CreateEditPoint();
			bool bHasSelection = true;
			if (startpoint.EqualTo(endpoint))
			{
				bHasSelection = false;
			}

			if (!(startpoint.AtStartOfLine))
			{
				startpoint.StartOfLine();
			}

			string strName = GetRegionName();
			if (Utilities.IsEmpty(strName))
			{
				return false;
			}

			bool bCloseUndoContext = false;
			if (!App.UndoContext.IsOpen)
			{
				bCloseUndoContext = true;
				App.UndoContext.Open("Menees.VSTools.RegionBuilder.MakeRegion", false);
			}
			try
			{
				string strPad = GetPadding(startpoint, bHasSelection);
				MakeRegion(Doc, strName, startpoint, endpoint, strPad, eLanguage, bHasSelection);
			}
			finally
			{
				if (bCloseUndoContext)
				{
					App.UndoContext.Close();
				}
			}

			return true;
		}

		#endregion

		#region Private Methods

		private static void MakeRegion(Document Doc, string strRegionName, EditPoint StartPoint, EditPoint EndPoint,
			string strPad, Language eLanguage, bool bHasSelection)
		{
			string strBeginRegionToken = "";
			string strEndRegionToken = "";
			string strRegionNameQuote = "";
			switch (eLanguage)
			{
				case Language.CSharp:
					strBeginRegionToken = "region";
					strEndRegionToken = "endregion";
					break;
				case Language.VB:
					strBeginRegionToken = "Region";
					strEndRegionToken = "End Region";
					strRegionNameQuote = "\"";
					break;
				case Language.CPlusPlus:
					strBeginRegionToken = "pragma region";
					strEndRegionToken = "pragma endregion";
					break;
			}

			string strStartRegion = "{0}#{3} {4}{1}{4}{2}{2}";
			string strEndRegion;
			if (!EndPoint.AtStartOfLine || !bHasSelection)
			{
				strEndRegion = "{1}{1}{0}#{2}";
			}
			else
			{
				strEndRegion = "{1}{0}#{2}{1}";
			}

			StartPoint.Insert(string.Format(strStartRegion, strPad, strRegionName, "\r\n", strBeginRegionToken, strRegionNameQuote));

			if (!bHasSelection)
			{
				//The insert statement above moved StartPoint to the end of the inserted text.
				//Since there was no selection now EndPoint is BEFORE StartPoint, so we need
				//to move it back after StartPoint.
				EndPoint = StartPoint;
			}

			EndPoint.Insert(string.Format(strEndRegion, strPad, "\r\n", strEndRegionToken));

			//If there was no selection, then move the caret back into
			//the middle of the new region.
			if (!bHasSelection)
			{
				EndPoint.LineUp(2);
				TextSelection Sel = (TextSelection)Doc.Selection;
				Sel.MoveToPoint(EndPoint, false);
			}
		}

		private static string GetRegionName()
		{
			string[] arPredefinedRegions = Settings.Default.SplitPredefinedRegions();
			string strResult = "";
			ListDlg Dlg = new ListDlg();
			Dlg.Execute("Add Region", "Enter a name or number:", "Region", arPredefinedRegions, ref strResult);
			return strResult;
		}

		private static string GetPadding(EditPoint StartPoint, bool bHasSelection)
		{
			string strTemp = StartPoint.GetText(StartPoint.LineLength);
			int iTempLength = strTemp.Length;

			StringBuilder SB = new StringBuilder();
			for (int i = 0; i < iTempLength; i++)
			{
				char ch = strTemp[i];
				if (ch == ' ' || ch == '\t')
				{
					SB.Append(ch);
				}
				else
				{
					break;
				}
			}

			//If there was no selection and no padding was calculated
			//and the current temp is blank, then use the padding from
			//the line above it.
			string strPad = SB.ToString();
			if (!bHasSelection && strPad.Length == 0 && iTempLength == 0)
			{
				EditPoint NewStartPoint = StartPoint.CreateEditPoint();
				NewStartPoint.LineUp(1);
				//Make sure we can still go up a line.
				if (NewStartPoint.Line == StartPoint.Line - 1)
				{
					strPad = GetPadding(NewStartPoint, bHasSelection);
				}
			}

			return strPad;
		}

		#endregion
	}
}
