﻿#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Copyright © 2007 Bill Menees.  All rights reserved.
	Bill@Menees.com

	$History: ToolWindow.cs $
	
	*****************  Version 1  *****************
	User: Bill         Date: 11/28/07   Time: 9:23p
	Created in $/CSharp/MeneesVSTools/2008

-----------------------------------------------------------------------------*/

#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using EnvDTE80;
using EnvDTE;
using System.Windows.Forms;
using stdole;
using System.Drawing;
using System.Runtime.InteropServices;

#endregion

namespace Menees.VSTools
{
	internal abstract class ToolWindow
	{
		#region Constructors

		protected ToolWindow(DTE2 application, AddIn addin, string windowGuid)
		{
			m_application = application;
			m_addin = addin;
			m_windowGuid = windowGuid;
		}

		#endregion

		#region Public Properties

		public abstract string Name { get; }

		public abstract Type UserControlType { get; }

		public abstract Image TabPicture { get; }

		public abstract string StateData { get; set; }

		#endregion

		#region Public Methods

		public void Initialize()
		{
			//Note: We need to use virtual properties, so we can't do this work in the constructor.

			//If the window was showing at the last shutdown, then we need to show it again.
			WindowState state = new WindowState(StateData);
			if (state.Visible)
			{
				Show(state);
			}
		}

		public void Show()
		{
			Show(null);
		}

		public void Close()
		{
			if (m_window != null)
			{
				WindowState state = new WindowState(m_window);
				StateData = state.StateData;
			}
		}

		#endregion

		#region Private Methods

		private void Show(WindowState state)
		{
			if (m_window == null)
			{
				m_window = CreateWindow();
				if (state == null)
				{
					state = new WindowState(StateData);
				}
			}

			if (m_window != null)
			{
				m_window.Visible = true;
				if (state != null)
				{
					state.Apply(m_window);
				}
			}
		}

		private Window CreateWindow()
		{
			Type type = UserControlType;
			object programmableObject = null;
			Windows2 windows2 = (Windows2)m_application.Windows;
			Window result = windows2.CreateToolWindow2(m_addin, type.Assembly.Location, type.FullName,
				Name, m_windowGuid, ref programmableObject);
			if (result != null)
			{
				//Set the tab icon for when the tool window is docked.
				//This must be done before we set Visible to true.
				result.SetTabPicture(ImageToPictureDispConverter.Convert(TabPicture));
			}

			//Note: If we need to get a reference to the UserControl instance, we can cast result.Object;
			return result;
		}

		#endregion

		#region Private Helper Types

		private class WindowState
		{
			#region Constructors

			public WindowState(string stateData)
			{
				m_stateData = stateData;
			}

			public WindowState(Window window)
			{
				StringBuilder sb = new StringBuilder();

				sb.Append(window.Visible ? "1" : "0");
				sb.Append(' ');
				sb.Append(window.Left);
				sb.Append(' ');
				sb.Append(window.Top);
				sb.Append(' ');
				sb.Append(window.Width);
				sb.Append(' ');
				sb.Append(window.Height);

				m_stateData = sb.ToString();
			}

			#endregion

			#region Public Properties

			public bool Visible
			{
				get
				{
					//This must match the string format built in the constructor.
					return !string.IsNullOrEmpty(m_stateData) && m_stateData.StartsWith("1");
				}
			}

			public string StateData
			{
				get
				{
					return m_stateData;
				}
			}

			#endregion

			#region Public Methods

			public void Apply(Window window)
			{
				if (!string.IsNullOrEmpty(m_stateData))
				{
					string[] values = m_stateData.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);

					//This must match the string format built in the constructor.
					if (values.Length == 5)
					{
						//We don't use values[0] here because we're required to set
						//the window visible before we can set the remaining properties.
						int left = Convert.ToInt32(values[1]);
						int top = Convert.ToInt32(values[2]);
						int width = Convert.ToInt32(values[3]);
						int height = Convert.ToInt32(values[4]);

						//Make sure the window is in the visible screen area.
						if (SystemInformation.VirtualScreen.Contains(left, top))
						{
							try
							{
								window.Visible = true;
								window.Left = left;
								window.Top = top;
								window.Width = width;
								window.Height = height;
							}
							catch (COMException ex)
							{
								if (ex.Message != "Error HRESULT E_FAIL has been returned from a call to a COM component.")
								{
									throw;
								}
							}
						}
					}
				}
			}

			#endregion

			#region Private Data Members

			private string m_stateData;

			#endregion
		}

		//This lovely class came from http://support.microsoft.com/kb/555417
		private class ImageToPictureDispConverter : AxHost
		{
			#region Constructors

			private ImageToPictureDispConverter() :
				base("{63109182-966B-4e3c-A8B2-8BC4A88D221C}")
			{
			}

			#endregion

			#region Public Methods

			public static IPictureDisp Convert(Image image)
			{
				ImageToPictureDispConverter instance = new ImageToPictureDispConverter();
				return instance.GetIPictureDispFromImage(image);
			}

			#endregion

			#region Private Methods

			private IPictureDisp GetIPictureDispFromImage(Image image)
			{
				IPictureDisp result = (IPictureDisp)GetIPictureDispFromPicture(image);
				return result;
			}

			#endregion
		}

		#endregion

		#region Private Data Members

		private DTE2 m_application;
		private AddIn m_addin;
		private string m_windowGuid;
		private Window m_window;

		#endregion
	}
}
